"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const axios_1 = __importDefault(require("axios"));
const common_1 = require("../common");
const system_1 = __importStar(require("../system"));
const Utils_1 = require("../system/Utils");
const Consts_1 = require("./Consts");
let lastAuthHeaders = -1;
let cachedAuthHeaders = {};
async function getAuthHeaders(useCachedHeaders = true) {
    if (process.env.NODE_ENV === 'test') {
        return {};
    }
    const timestamp = Date.now();
    if (useCachedHeaders) {
        if (timestamp - lastAuthHeaders < 300000) {
            return cachedAuthHeaders;
        }
    }
    lastAuthHeaders = timestamp;
    const auth = system_1.default.getLibModule(system_1.LibModule.AUTH);
    const organization = await auth.getOrganization();
    cachedAuthHeaders = {
        [Consts_1.X_ORGANIZATION]: organization.module_id,
        [Consts_1.X_CLIENT_TZ]: Intl.DateTimeFormat().resolvedOptions().timeZone,
        'Content-Type': 'application/json;charset=UTF-8',
    };
    return cachedAuthHeaders;
}
function rejectedBecauseAuth(e) {
    return Boolean(e.response && e.response.status && (e.response.status === Consts_1.FORBIDDEN || e.response.status === Consts_1.UNAUTHORIZED));
}
/**
 * Makes sure that a URI contains no leading '/' characters
 *
 * @param uri The URI being sanitized
 */
function sanitizeUri(uri) {
    const length = uri.length;
    let i = 0;
    for (; i < length; i++) {
        if (uri.charAt(i) !== '/') {
            break;
        }
    }
    if (i === 0) {
        return uri;
    }
    return uri.substr(i);
}
async function request(method, uri, data, headers, canRetryAuth = true) {
    let authFailed = false;
    const requestSettings = {
        method,
        withCredentials: true,
    };
    if (data) {
        if (method === Consts_1.GET || method === Consts_1.DELETE) {
            requestSettings.params = data;
        }
        else {
            requestSettings.data = data;
        }
    }
    const authHeaders = await getAuthHeaders(canRetryAuth);
    requestSettings.headers = { ...authHeaders, ...system_1.default.getHttpHeaders(), ...headers };
    try {
        const url = `${system_1.default.nexus.getUrl()}/${sanitizeUri(uri)}`;
        const response = (await axios_1.default(url, requestSettings));
        if (!response || !response.data) {
            return null;
        }
        return response.data;
    }
    catch (e) {
        authFailed = rejectedBecauseAuth(e);
    }
    if (authFailed && canRetryAuth) {
        // TODO: consider setting an amount of times to retry auth
        const user = await system_1.default.nexus.reconnect();
        if (user) {
            return request(method, uri, data, headers, false);
        }
    }
    return null;
}
function get(uri, params, headers) {
    return request(Consts_1.GET, uri, params, headers);
}
function del(uri, params, headers) {
    return request(Consts_1.DELETE, uri, params, headers);
}
function post(uri, payload, headers) {
    return request(Consts_1.POST, uri, payload, headers);
}
function put(uri, payload, headers) {
    return request(Consts_1.PUT, uri, payload, headers);
}
async function search(domain, module, filters, options) {
    common_1.validateDomainAndModule(domain, module);
    const domainUri = common_1.domainToUri(domain);
    const moduleUri = common_1.moduleToUri(module);
    const response = await request(Consts_1.PUT, `${domainUri}/${moduleUri}/search`, { ...options, filters }, options?.headers);
    return response ? response : [];
}
const api = Object.freeze(Utils_1.makeCallable(request, { get, delete: del, post, put, search }));
exports.default = system_1.default.sealModule(api);
