"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const common_1 = require("../common");
const system_1 = __importStar(require("../system"));
const Utils_1 = require("../system/Utils");
function entityUri(domain, module, text) {
    const domainUri = common_1.domainToUri(domain);
    const moduleUri = common_1.moduleToUri(module);
    if (text) {
        return `${domainUri}/${moduleUri}/entity/${text}`;
    }
    return `${domainUri}/${moduleUri}/entity`;
}
function externalEntityUri(moduleName, text) {
    const moduleNameUri = common_1.toLowerCamel(moduleName);
    if (text) {
        return `core/entity/external/${moduleNameUri}/${text}`;
    }
    return `core/entity/external/${moduleNameUri}`;
}
function baseGet(api, domain, module, id, headers) {
    const uri = entityUri(domain, module, id);
    return api.get(uri, undefined, headers);
}
async function baseBulkCreate(api, domain, module, data, headers) {
    const bulkCreateUri = entityUri(domain, module, 'bulkCreate');
    const response = await api.post(bulkCreateUri, data, headers);
    return response ? response : [];
}
function baseCreate(api, domain, module, data, headers) {
    const uri = entityUri(domain, module, 'create');
    return api.post(uri, data, headers);
}
function baseDelete(api, domain, module, id, data, headers) {
    const uri = entityUri(domain, module, 'delete');
    return api.post(uri, { ...data, id }, headers);
}
function baseSearch(api, domain, module, filters, options) {
    return api.search(domain, module, filters, options);
}
function baseView(api, domain, module, id, data, headers) {
    const uri = entityUri(domain, module, 'view');
    return api.post(uri, { ...data, id }, headers);
}
function entity(domain, module) {
    const entityDomain = domain;
    const entityModule = module;
    common_1.validateDomainAndModule(entityDomain, entityModule);
    const api = system_1.default.getLibModule(system_1.LibModule.API);
    function get(id, headers) {
        return baseGet(api, entityDomain, entityModule, id, headers);
    }
    function bulkCreate(data, headers) {
        return baseBulkCreate(api, entityDomain, entityModule, data, headers);
    }
    function create(data, headers) {
        return baseCreate(api, entityDomain, entityModule, data, headers);
    }
    function del(id, data, headers) {
        return baseDelete(api, entityDomain, entityModule, id, data, headers);
    }
    function search(filters, options) {
        return baseSearch(api, entityDomain, entityModule, filters, options);
    }
    function view(id, data, headers) {
        return baseView(api, entityDomain, entityModule, id, data, headers);
    }
    return Object.freeze({
        get,
        bulkCreate,
        create,
        delete: del,
        search,
        view,
    });
}
const addCustomAttributeUri = 'core/entity/external/customAttribute/add';
const removeCustomAttributeUri = 'core/entity/external/customAttribute/remove';
const updateCustomAttributeUri = 'core/entity/external/customAttribute/modify';
function external(moduleName) {
    const api = system_1.default.getLibModule(system_1.LibModule.API);
    const bulkCreateUri = externalEntityUri(moduleName, 'bulkCreate');
    const createUri = externalEntityUri(moduleName, 'create');
    const updateUri = externalEntityUri(moduleName, 'update');
    const searchUri = 'search';
    function get(id, headers) {
        const uri = externalEntityUri(moduleName, id);
        return api.get(uri, undefined, headers);
    }
    async function bulkCreate(data, headers) {
        const response = await api.post(bulkCreateUri, data, headers);
        return response ? response : [];
    }
    function create(data, headers) {
        return api.post(createUri, data, headers);
    }
    function update(data, headers) {
        return api.post(updateUri, data, headers);
    }
    async function search(filters, options) {
        const response = await api.put(searchUri, { ...options, filters, module_name: moduleName });
        return response ? response : [];
    }
    function addCustomAttribute(moduleId, attribute, headers) {
        const payload = {
            module: { module_name: moduleName, module_id: moduleId },
            custom_attribute: attribute,
        };
        return api.post(addCustomAttributeUri, payload, headers);
    }
    function removeCustomAttribute(moduleId, attributeId, headers) {
        const payload = {
            module: { module_name: moduleName, module_id: moduleId },
            custom_attribute_id: attributeId,
        };
        return api.post(removeCustomAttributeUri, payload, headers);
    }
    function updateCustomAttribute(moduleId, attribute, headers) {
        const payload = {
            module: { module_name: moduleName, module_id: moduleId },
            custom_attribute: attribute,
        };
        return api.post(updateCustomAttributeUri, payload, headers);
    }
    return Object.freeze({
        get,
        bulkCreate,
        create,
        update,
        search,
        addCustomAttribute,
        removeCustomAttribute,
        updateCustomAttribute,
    });
}
function internalGet(domain, module, id) {
    common_1.validateDomainAndModule(domain, module);
    return baseGet(system_1.default.getLibModule(system_1.LibModule.API), domain, module, id);
}
function internalCreate(domain, module, data) {
    common_1.validateDomainAndModule(domain, module);
    return baseCreate(system_1.default.getLibModule(system_1.LibModule.API), domain, module, data);
}
function internalDelete(domain, module, id, data) {
    common_1.validateDomainAndModule(domain, module);
    return baseDelete(system_1.default.getLibModule(system_1.LibModule.API), domain, module, id, data);
}
function internalView(domain, module, id, data) {
    common_1.validateDomainAndModule(domain, module);
    return baseView(system_1.default.getLibModule(system_1.LibModule.API), domain, module, id, data);
}
const moduleInstance = Object.freeze(Utils_1.makeCallable(entity, {
    get: internalGet,
    create: internalCreate,
    delete: internalDelete,
    view: internalView,
    external,
}));
exports.default = system_1.default.sealModule(moduleInstance);
