"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
async function nexus(system, config, clientAuth) {
    let user = null;
    let baseUrl = config.url;
    if (baseUrl[baseUrl.length - 1] === '/') {
        baseUrl = baseUrl.slice(0, -1);
    }
    function getUrl() {
        return baseUrl;
    }
    function getUser() {
        return user;
    }
    function getLoginUrl(returnUrl) {
        if (typeof returnUrl !== 'string') {
            return `${baseUrl}/login`;
        }
        return `${baseUrl}/login?returnUrl=${returnUrl}`;
    }
    async function disconnect() {
        await clientAuth.disconnect(system);
        if (clientAuth.afterDisconnect) {
            clientAuth.afterDisconnect(system);
        }
    }
    async function reconnect() {
        await disconnect();
        const result = await clientAuth.reconnect(system);
        if (!result) {
            return null;
        }
        user = result;
        if (clientAuth.afterConnect) {
            clientAuth.afterConnect(system);
        }
        return result;
    }
    const instance = {
        getUrl,
        getLoginUrl,
        disconnect,
        reconnect,
    };
    try {
        user = await clientAuth.connect(baseUrl);
        if (!user) {
            throw new Error('was not able to authenticate the user');
        }
        return Object.freeze({ ...instance, getUser });
    }
    catch (e) {
        throw new Error(`failed to establish a proper connection with a nexus node: ${e.message}`);
    }
}
exports.default = nexus;
